Package: arm
Version: 1.15-3
Date: 2026-4-15
Title: Data Analysis Using Regression and Multilevel/Hierarchical
        Models
Authors@R: c(person("Andrew", "Gelman", role = "aut", email = "gelman@stat.columbia.edu"),
             person("Yu-Sung", "Su", role =  c("aut", "cre"), 
		          email = "suyusung@tsinghua.edu.cn", 
		          comment = c(ORCID = "0000-0001-5021-8209")),
             person("Masanao", "Yajima", role =  "ctb", email = "yajima@bu.edu"),
             person("Jennifer", "Hill", role =  "ctb", email = "jennifer.hill@nyu.edu"),
             person("Maria Grazia", "Pittau", role =  "ctb", email = "grazia@stat.columbia.edu"),
             person("Jouni", "Kerman", role =  "ctb", email = "jouni@kerman.com"),
             person("Tian", "Zheng", role =  "ctb", email = "tzheng@stat.columbia.edu"),
             person("Vincent", "Dorie", role =  "ctb", email = "vjd4@nyu.edu")
             )
Author: Andrew Gelman [aut],
  Yu-Sung Su [aut, cre] (ORCID: <https://orcid.org/0000-0001-5021-8209>),
  Masanao Yajima [ctb],
  Jennifer Hill [ctb],
  Maria Grazia Pittau [ctb],
  Jouni Kerman [ctb],
  Tian Zheng [ctb],
  Vincent Dorie [ctb]
Maintainer: Yu-Sung Su <suyusung@tsinghua.edu.cn>
BugReports: https://github.com/suyusung/arm/issues/
Depends: R (>= 3.1.0), MASS, Matrix (>= 1.6-1.1), stats, lme4 (>= 1.0)
Imports: abind, coda, graphics, grDevices, methods, nlme, utils
Description: Functions to accompany A. Gelman and J. Hill, Data Analysis Using Regression and Multilevel/Hierarchical Models, Cambridge University Press, 2007.
License: GPL (>= 2)
URL: https://CRAN.R-project.org/package=arm
NeedsCompilation: no
Packaged: 2026-04-15 11:46:23 UTC; suyusung
Repository: CRAN
Date/Publication: 2026-04-15 14:00:02 UTC
Built: R 4.5.2; ; 2026-04-17 09:21:44 UTC; unix
